dnl
dnl
dnl This will check if ipv6 is available
dnl
dnl

AC_DEFUN([AC_CHECK_INET6], [
AC_MSG_CHECKING([if ipv6 is available])
AC_TRY_RUN([
#include <sys/types.h>
#include <sys/socket.h>
int main(void)
{
  if (socket(AF_INET6, SOCK_STREAM, 0) < 0) {
    return(1);
  }
  else {
    return(0);
  }

  return(0);
}
  ],
  [
    AC_DEFINE(ENABLE_INET6)
    ipv6=yes
  ],
  [
    ipv6=no
  ],
  [
    ipv6=no
  ])
AC_MSG_RESULT($ipv6)

ipv6type=unknown
ipv6lib=none
ipv6trylibc=yes

if test "$ipv6" = "yes"; then
  AC_MSG_CHECKING([ipv6 stack type])
    for i in inria kame linux-glibc linux-inet6 toshiba v6d zeta solaris; do
      case $i in
        inria)
        # http://www.kame.net/
        AC_EGREP_CPP(yes, [
#include <netinet/in.h>
#ifdef IPV6_INRIA_VERSION
yes
#endif],
          [
            ipv6type=$i
            CFLAGS="-DINET6 $CFLAGS"
          ])
        ;;

      kame)
        # http://www.kame.net/
        AC_EGREP_CPP(yes, [
#include <netinet/in.h>
#ifdef __KAME__
yes
#endif],
          [
            ipv6type=$i
            ipv6lib=inet6
            ipv6libdir=/usr/local/v6/lib
            ipv6trylibc=yes
            CFLAGS="-DINET6 $CFLAGS"
          ])
        ;;

      linux-glibc)
        # http://www.v6.linux.or.jp/
        AC_EGREP_CPP(yes, [
#include <features.h>
#if defined(__GLIBC__) && __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 1
yes
#endif],
          [
            ipv6type=$i;
            CFLAGS="-DINET6 $CFLAGS"
          ])
        ;;

      linux-inet6)
        # http://www.v6.linux.or.jp/
        if test -d /usr/inet6; then
          ipv6type=$i
          ipv6lib=inet6
          ipv6libdir=/usr/inet6/lib
          CFLAGS="-DINET6 -I/usr/inet6/include $CFLAGS"
        fi
        ;;

      toshiba)
        AC_EGREP_CPP(yes, [
#include <sys/param.h>
#ifdef _TOSHIBA_INET6
yes
#endif],
          [
            ipv6type=$i
            ipv6lib=inet6
            ipv6libdir=/usr/local/v6/lib
            CFLAGS="-DINET6 $CFLAGS"
          ])
        ;;

      v6d)
        AC_EGREP_CPP(yes, [
#include </usr/local/v6/include/sys/v6config.h>
#ifdef __V6D__
yes
#endif],
          [
            ipv6type=$i
            ipv6lib=v6
            ipv6libdir=/usr/local/v6/lib
            CFLAGS="-I/usr/local/v6/include $CFLAGS"
          ])
        ;;

      zeta)
        AC_EGREP_CPP(yes, [
#include <sys/param.h>
#ifdef _ZETA_MINAMI_INET6
yes
#endif],
          [
            ipv6type=$i
            ipv6lib=inet6
            ipv6libdir=/usr/local/v6/lib
             CFLAGS="-DINET6 $CFLAGS"
          ])
        ;;

      solaris)
        AC_EGREP_CPP(yes, [
#include <sys/elf_notes.h>
#ifdef ELF_NOTE_SOLARIS
yes
#endif],
          [
            ipv6type=$i
          ])
        ;;

      esac
    if test "$ipv6type" != "unknown"; then
      break
    fi
  done
  AC_MSG_RESULT($ipv6type)
fi

if test "$ipv6" = "yes" -a "$ipv6lib" != "none"; then
  if test -d $ipv6libdir -a -f $ipv6libdir/lib$ipv6lib.a; then
    LIBS="-L$ipv6libdir -l$ipv6lib $LIBS"
    echo "You have $ipv6lib library, using it"
  else
    if test "$ipv6trylibc" = "yes"; then
      echo "You do not have $ipv6lib library, using libc"
    else
      echo 'Fatal: no $ipv6lib library found.  cannot continue.'
      echo "You need to fetch lib$ipv6lib.a from appropriate"
      echo 'ipv6 kit and compile beforehand.'
      exit 1
    fi
  fi
fi
])
])
AC_DEFUN([AC_DVTS_CHECK_LINUX_SSFAMILY],
[
  linux_ssfamily=no
  AC_MSG_CHECKING([if struct sockaddr_storage has __ss_family])
  AC_TRY_RUN([
#include <sys/types.h>
#include <sys/socket.h>
int main(void)
{
  struct sockaddr_storage ss;
  ss.__ss_family = 0;
  return(0);
}
  ],
  [
    CFLAGS="-DLINUX_OLD_SS_FAMILY $CFLAGS"
    linux_ssfamily=yes
  ],
  [
    linux_ssfamily=no
  ],
  [
    linux_ssfamily=no
  ])

AC_MSG_RESULT($linux_ssfamily)

])
dnl @synopsis AC_PROG_CC_WARNINGS([ANSI])
dnl
dnl Enables a reasonable set of warnings for the C compiler.  Optionally,
dnl if the first argument is nonempty, turns on flags which enforce and/or
dnl enable proper ANSI C if such flags are known to the compiler used.
dnl
dnl Currently this macro knows about GCC, Solaris C compiler,
dnl Digital Unix C compiler, C for AIX Compiler, HP-UX C compiler,
dnl and IRIX C compiler.
dnl
dnl @version $Id: acinclude.m4,v 1.2 2004/06/30 17:13:20 hopet Exp $
dnl @author Ville Laurikari <vl@iki.fi>
dnl
AC_DEFUN([AC_PROG_CC_WARNINGS], [
  ansi=$1
  if test -z "$ansi"; then
    msg="for C compiler warning flags"
  else
    msg="for C compiler warning and ANSI conformance flags"
  fi
  AC_CACHE_CHECK($msg, ac_cv_prog_cc_warnings, [
    if test -n "$CC"; then
      cat > conftest.c <<EOF
int main(int argc, char **argv) { return 0; }
EOF

      dnl GCC
      if test "$GCC" = "yes"; then
        if test -z "$ansi"; then
          ac_cv_prog_cc_warnings="-Wall"
        else
          ac_cv_prog_cc_warnings="-Wall -ansi"
        fi

      dnl Solaris C compiler
      elif $CC -flags 2>&1 | grep "Xc.*strict ANSI C" > /dev/null 2>&1 &&
           $CC -c -v -Xc conftest.c > /dev/null 2>&1 &&
           test -f conftest.o; then
        if test -z "$ansi"; then
          ac_cv_prog_cc_warnings="-v"
        else
          ac_cv_prog_cc_warnings="-v -Xc"
        fi

      dnl HP-UX C compiler
      elif $CC > /dev/null 2>&1 &&
           $CC -c -Aa +w1 conftest.c > /dev/null 2>&1 &&
           test -f conftest.o; then
        if test -z "$ansi"; then
          ac_cv_prog_cc_warnings="+w1"
        else
          ac_cv_prog_cc_warnings="+w1 -Aa"
        fi

      dnl Digital Unix C compiler
      elif ! $CC > /dev/null 2>&1 &&
           $CC -c -verbose -w0 -warnprotos -std1 conftest.c > /dev/null 2>&1 &&
           test -f conftest.o; then
        if test -z "$ansi"; then
          ac_cv_prog_cc_warnings="-verbose -w0 -warnprotos"
        else
          ac_cv_prog_cc_warnings="-verbose -w0 -warnprotos -std1"
        fi

      dnl C for AIX Compiler
      elif $CC > /dev/null 2>&1 | grep AIX > /dev/null 2>&1 &&
           $CC -c -qlanglvl=ansi -qinfo=all conftest.c > /dev/null 2>&1 &&
           test -f conftest.o; then
        if test -z "$ansi"; then
          ac_cv_prog_cc_warnings="-qsrcmsg -qinfo=all:noppt:noppc:noobs:nocnd"
        else
          ac_cv_prog_cc_warnings="-qsrcmsg -qinfo=all:noppt:noppc:noobs:nocnd -qlanglvl=ansi"
        fi

      dnl IRIX C compiler
      elif $CC -fullwarn -ansi -ansiE > /dev/null 2>&1 &&
           test -f conftest.o; then
        if test -z "$ansi"; then
          ac_cv_prog_cc_warnings="-fullwarn"
        else
          ac_cv_prog_cc_warnings="-fullwarn -ansi -ansiE"
        fi

      fi
      rm -f conftest.*
    fi
    if test -n "$ac_cv_prog_cc_warnings"; then
      CFLAGS="$CFLAGS $ac_cv_prog_cc_warnings"
    else
      ac_cv_prog_cc_warnings="unknown"
    fi
  ])
])
AC_DEFUN([AC_WITH_LIBDV], [
  AC_ARG_WITH(libdv-path,
    [  --with-libdv-path=DIR      path to libdv.],
    [libdvpath="$withval"])

  CFLAGS="$CFLAGS -I$libdvpath/include"
  LDFLAGS="$LDFLAGS -L$libdvpath/lib"

  LIBS="$LIBS -ldv"
])
AC_DEFUN([AC_WITH_SDL], [
  AC_ARG_WITH(sdl,
    [  --with-sdl      use SDL.
  --without-sdl      do not use SDL.],
    [use_sdl=$withval])
])
AC_DEFUN([AC_WITH_XV], [
  AC_ARG_WITH(xv,
    [  --with-xv      use libXv.
  --without-xv      do not use libXv.],
    [use_xv=$withval])
])
