/* We need thread-safe sem_wait that works on FreeBSD as well as on Linux */
/* This is to avoid problem with FreeBSD/KSE (5.2.1) getting stuck in sem_wait
 * call; sem_wait doesn't seem to be pthread_testcancel() point there :-(( */
#ifndef FREEBSD_5 
#define ts_sem_wait(semaphore) \
			if(sem_wait(semaphore) != 0) { \
				perror("sem_wait"); \
				assert(0); \
			}
#else
#define ts_sem_wait(semaphore) \
			while(sem_trywait(semaphore) != 0) { \
				if(errno != EAGAIN) { \
					perror("sem_trywait"); \
					assert(0); \
				} \
				usleep(1); \
			}
#endif /* FREEBSD_5 */ 

/* I wonder why the heck the damned libdv/dv_types.h header does the following:
#define __FUNCTION__ __FILE__ // Less specific info, but it's a string.
*/
#undef __FUNCTION__
#ifdef DEBUG
#define DPRINT(fmt, args...) \
	fprintf(stderr, "(%s:%d) %s(): " fmt, __FILE__, __LINE__ , __FUNCTION__ , ##args); fflush(stderr);
#define DPRINT_BARE(fmt, args...) \
	fprintf(stderr, fmt, ##args); fflush(stderr);
#else
#define DPRINT(fmt, args...)	((void)0)
#define DPRINT_BARE(fmt, args...)	((void)0)
#endif /* DEBUG */
