#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>
#include <string.h>
#include <errno.h>

#include <semaphore.h>

#if (LINUX && HAVE_LIBRAW1394)
#include <libraw1394/raw1394.h>
#include <libraw1394/csr.h>
#endif /* LINUX && HAVE_LIBRAW1394 */

#ifdef FREEBSD_5
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <err.h>

#include <dev/firewire/firewire.h>
#include "iec68113.h"
#endif /* FREEBSD_5 */

#include "xdvshow-shm.h"
#include "xdvshow-flags.h"
#include "xdvshow-ieee1394.h"
#include "xdvshow-const.h"
#include "xdvshow-defs.h"

#if (LINUX && HAVE_LIBRAW1394)

#define RAW_BUF_SIZE    (10240)
#define RAW_PACKET_MAX	4096		/* XXX What the hell is this? */

int g_card = 0;
int g_channel = 63;
int g_frame_count = 100000;
volatile int g_alldone = 0;
volatile int g_reader_active;

unsigned char *g_frame;

raw1394handle_t open_1394_driver(int channel, raw1394_iso_recv_handler_t handler);
void close_1394_driver(raw1394handle_t handle);

#endif /* LINUX && HAVE_LIBRAW1394 */

#ifdef FREEBSD_5

char *system_name[] = {"NTSC", "PAL"};
int frame_rate[] = {30, 25};

#define PSIZE 512
#define DSIZE 480
#define NCHUNK 8

#define NPACKET_R 256
#define NPACKET_T 255

#define NPKT 50
#define NVEC 50
#define	BLOCKSIZE 80

#define TNBUF 100       /* XXX too large value causes block noise */
#define NEMPTY 10       /* depends on TNBUF */

#define RBUFSIZE (PSIZE * NPACKET_R)
#define MAXBLOCKS (300)
#define CYCLE_FRAC 0xc00

#endif /* FREEBSD_5 */


#if (LINUX && HAVE_LIBRAW1394)

/* Switch to new API */
/*
int raw_iso_handler(raw1394handle_t handle, int channel, size_t length,
                    quadlet_t *data)
*/
static enum raw1394_iso_disposition
raw_iso_handler(raw1394handle_t handle, unsigned char *data,
	unsigned int length, unsigned char channel,
	unsigned char tag, unsigned char sy, unsigned int cycle,
	unsigned int dropped)
{
        if (length < RAW_BUF_SIZE) {
                *(int*)g_frame = length;
                memcpy(g_frame + 4, data, length);
        }
        return RAW1394_ISO_OK;
}

void *
xdvshow_capture_raw(void *data)
{
        int frames_read;
        int length;
        raw1394handle_t handle;
        int found_first_frame;
        int skipped = 0;

	int offset = 0;
	int ret;

	u_char *dvdata;
	u_char *audio_dvdata;

#ifdef DEBUG
        int sem_value = 0;
	int iDebugSemVal;
#endif /* DEBUG */

        ret = _xdvshow_alloc_shm();
        if (ret < 0) {
          return((void *)1);
        }

        ret = _xdvshow_attach_shm();

	dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
	audio_dvdata = _xdvshow_shm.audio_shm_frame->frame_buf->data;
	
        g_frame = (unsigned char*)malloc(RAW_BUF_SIZE);

        handle = open_1394_driver(g_channel, raw_iso_handler);

        frames_read = 0;
        found_first_frame = 0;

	raw1394_iso_recv_start(handle, -1, -1, 0);
	
	printf("raw1394 - start capture\n");
        while (!g_alldone) {
                raw1394_loop_iterate(handle);
                length = *(int*)g_frame;
                if (length >= 492) {
                        if (!found_first_frame) {
                                if (g_frame[16] == 0x1f && g_frame[17] == 0x07)
                                        found_first_frame = 1;
                                else skipped++;
                        }
                        if (skipped > 500) {
                                printf("skipped too many without finding frame\n");
                                break;
                        }
                        if (found_first_frame) {
                                if (g_frame[16] == 0x1f && g_frame[17] == 0x07)
                                        frames_read++;
                                if (frames_read > g_frame_count)
                                        break;
			        if (offset < 144000){
			          memcpy(&dvdata[offset], (g_frame + 16), 480);
			          offset += 480;
				}
			        if (offset == 144000){
			          offset = 0;

			          _xdvshow_shm.shm_frame->frame_buf->lock = DVFRAME_DATA_READY;

#ifdef DEBUG
					DPRINT("RECEIVED frame no. %d\n", sem_value++);
					if(sem_getvalue(&video_ready, &iDebugSemVal)) {
						perror("Unable to get value of video_ready semaphore!");
						exit(EXIT_FAILURE);
					}
					DPRINT("semaphore video_ready value: %d\n", iDebugSemVal);

					if(sem_getvalue(&video_empty, &iDebugSemVal)) {
						perror("Unable to get value of video_empty semaphore!");
						exit(EXIT_FAILURE);
					}
					DPRINT("semaphore video_empty value: %d\n", iDebugSemVal);
#endif /* DEBUG */

					if (sem_post(&video_ready) < 0)
						perror("sem_post");
					ts_sem_wait(&video_empty);

		      	          _xdvshow_shm.shm_frame = _xdvshow_shm.shm_frame->next;
		                  dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
				}
                        }
                }
        }
        close_1394_driver(handle);
	
        free(g_frame);

        return 0;
}

/* Switch to new API */
/*
int
my_reset_handler(raw1394handle_t handle, unsigned int generation)
{
        static int i = 0;

        printf("reset %d\n", i++);
        if (i == 100)
                g_reader_active = 0;
        return 0;
}
*/


/* XXX - what is the correct type for handler: iso_handler_t? */
raw1394handle_t
open_1394_driver(int channel, raw1394_iso_recv_handler_t handler)
{
        int numcards;
        struct raw1394_portinfo g_pinf[16];
        raw1394handle_t handle;

        if (!(handle = raw1394_new_handle())) {
                perror("raw1394 - couldn't get handle");
                printf("This error usually means that the ieee1394 driver is not ready\n");
                exit( -1);
        }

        if ((numcards = raw1394_get_port_info(handle, g_pinf, 16)) < 0) {
                perror("raw1394 - couldn't get card info");
                exit( -1);
        }

        if (raw1394_set_port(handle, g_card) < 0) {
                perror("raw1394 - couldn't set port");
                exit( -1);
        }

	/* Switch to new API */
	/* raw1394_set_iso_handler(handle, g_channel, handler);		*/
        /* raw1394_set_tag_handler(handle, my_tag_handler);		*/
        /* raw1394_set_bus_reset_handler(handle, my_reset_handler);	*/

        /* Starting iso receive */

        /* Switch to new API */
	/*
	if (raw1394_start_iso_rcv(handle, channel) < 0) {
	
                perror("raw1394 - couldn't start iso receive");
                exit( -1);
        }
	*/

	/* XXX - This completely sucks :(( There should be a following call (according to libraw 1.1.0 doc):
	raw1394_iso_recv_init(handle, handler, RAW_BUF_SIZE, RAW_PACKET_MAX, channel, mode, -1);

	there's a mess in the libraw1394 versioning and the API changes with almost each new version :((

	This call is 1.1.0 (1.2.0) compatible.
	*/

	raw1394_iso_recv_init(handle, handler, RAW_BUF_SIZE, RAW_PACKET_MAX, channel, RAW1394_DMA_PACKET_PER_BUFFER, -1);
	printf("raw1394 - driver opened\n");
        return handle;
}

/** \brief Internal function used for closing the device via libraw1394 linux library.
 *
 *
 */

void
close_1394_driver(raw1394handle_t handle)
{
	/* Switch to new API */
  	/*
        raw1394_stop_iso_rcv(handle, channel);
        raw1394_destroy_handle(handle);
	*/

  	raw1394_iso_shutdown(handle);
	raw1394_destroy_handle(handle);
}

#endif /* LINUX && HAVE_LIBRAW1394 */

#ifdef FREEBSD_5

/** \brief xdvshow_capture_raw is called as the DV stream capturing thread under FreeBSD.
 *
 *
 *
 */

void *
xdvshow_capture_raw(void *data)
{
	char devname[] = "/dev/fw0";
 	int fd;
       	
  	struct fw_isochreq isoreq;
	struct fw_isobufreq bufreq;
	struct dvdbc *dv;
	struct ciphdr *ciph;
	struct fw_pkt *pkt;
	char *pad, *buf;
	u_int32_t *ptr;
	int len, tlen, k, m, vec, nb, system = -1 ;
#if FIX_FRAME
	int npad;
#endif /* FIX_FRAME */
	int nblocks[] = {250 /* NTSC */, 300 /* PAL */};
	struct iovec wbuf[NPACKET_R];

        int iovec_iter;

	int offset = 0;
	int ret;
	
	u_char *dvdata;
	u_char *audio_dvdata;

#ifdef DEBUG
        int sem_value = 0;
	int iDebugSemVal;
#endif /* DEBUG */

#if 0
	int mode;
#endif

	fd = open(devname, O_RDWR);
	if (fd < 0)
		err(1, "open");

	buf = (char *)malloc(RBUFSIZE);
	pad = (char *)malloc(DSIZE*MAXBLOCKS);
	
	memset(pad, 0xff, DSIZE*MAXBLOCKS);
	bzero(wbuf, sizeof(wbuf));


	bufreq.rx.nchunk = NCHUNK;
	bufreq.rx.npacket = NPACKET_R;
	bufreq.rx.psize = PSIZE;
	bufreq.tx.nchunk = 0;
	bufreq.tx.npacket = 0;
	bufreq.tx.psize = 0;
	if (ioctl(fd, FW_SSTBUF, &bufreq) < 0) {
		err(1, "%s", "ioctl");
	}

	isoreq.ch = ((1<<6) | 63) & 0x3f;
	isoreq.tag = (((1<<6) | 63) >> 6) & 3;

	if( ioctl(fd, FW_SRSTREAM, &isoreq) < 0)
       		err(1, "%s", "ioctl");

	k = m = 0;

        ret = _xdvshow_alloc_shm();
        if (ret < 0) {
          return((void *)1);
        }

	ret = _xdvshow_attach_shm();

	dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
	audio_dvdata = _xdvshow_shm.audio_shm_frame->frame_buf->data;
	
	while (1) {
#if 0
                tlen = 0;
                while ((len = read(d, buf + tlen, PSIZE)) > 0) {
                        if (len < 0) {
                                if (errno == EAGAIN) {
                                        fprintf(stderr, "(EAGAIN)\n");
                                        fflush(stderr);
                                        if (len <= 0)
                                                continue;
                                } else
                                        err(1, "read failed");
                        }
                        tlen += len;
                        if ((RBUFSIZE - tlen) < PSIZE)
                                break;
                };
#else
                tlen = len = read(fd, buf, RBUFSIZE);
                if (len < 0) {
                        if (errno == EAGAIN) {
                                fprintf(stderr, "(EAGAIN)\n");
                                fflush(stderr);
                                if (len <= 0)
                                        continue;
                        } else
                                err(1, "read failed");
                }
#endif
	
		vec = 0;
		ptr = (u_int32_t *) buf;
again:
		pkt = (struct fw_pkt *) ptr;

#if DEBUG
		DPRINT("%08x %08x %08x %08x\n", htonl(ptr[0]), htonl(ptr[1]), htonl(ptr[2]), htonl(ptr[3]));
#endif /* DEBUG */

		ciph = (struct ciphdr *)(ptr + 1);	/* skip iso header */
                if (ciph->fmt != CIP_FMT_DVCR)
                        errx(1, "unknown format 0x%x", ciph->fmt);
		ptr = (u_int32_t *) (ciph + 1);		/* skip cip header */

#if DEBUG
                if (ciph->fdf.dv.cyc != 0xffff && k == 0) { 
                        DPRINT("0x%04x\n", ntohs(ciph->fdf.dv.cyc));
                }
#endif /* DEBUG */

		if (pkt->mode.stream.len <= sizeof(struct ciphdr)) {
                        /* no payload */
			goto next;
		}
		
		for (dv = (struct dvdbc *)ptr; (char *)dv < (char *)(ptr + ciph->len); dv+=6) {
#if DEBUG
			DPRINT("(%d,%d) ", dv->sct, dv->dseq);
#endif /* DEBUG */
			if  (dv->sct == DV_SCT_HEADER && dv->dseq == 0) {
                                if (system < 0) {
                                        system = ciph->fdf.dv.fs;
                                        printf("%s\n", system_name[system]);
                                }

                                /* Fix DSF bit */
                                if (system == 1 && (dv->payload[0] & DV_DSF_12) == 0)
                                        dv->payload[0] |= DV_DSF_12;
                                nb = nblocks[system];
                                fprintf(stderr, "%d", k%10);

#if FIX_FRAME
				if (m > 0 && m != nb) {
					/* padding bad frame */
					npad = ((nb - m) % nb);
					if (npad < 0)
						npad += nb;
					printf("(%d blocks padded)", npad);
					npad *= DSIZE;
					wbuf[vec].iov_base = pad;
					wbuf[vec++].iov_len = npad;

					if (vec >= NPACKET_R) {
					  memcpy(&dvdata, pad, npad * BLOCKSIZE);
					  offset += npad * BLOCKSIZE;

					  vec = 0;
					  
					  if (offset == 144000) {
					    offset = 0;

					    _xdvshow_shm.shm_frame->frame_buf->lock = DVFRAME_DATA_READY;

#ifdef DEBUG
					    DPRINT("READ frame no. %d\n", sem_value++);
						if(sem_getvalue(&video_ready, &iDebugSemVal)) {
							perror("Unable to get value of video_ready semaphore!");
							exit(EXIT_FAILURE);
						}
						DPRINT("(with FIX_FRAME) semaphore video_ready value: %d\n", iDebugSemVal);

						if(sem_getvalue(&video_empty, &iDebugSemVal)) {
							perror("Unable to get value of video_empty semaphore!");
							exit(EXIT_FAILURE);
						}
						DPRINT("(with FIX_FRAME) semaphore video_empty value: %d\n", iDebugSemVal);
#endif /* DEBUG */

					    sem_post(&video_ready);
						ts_sem_wait(&video_empty);
					    
					    _xdvshow_shm.shm_frame = _xdvshow_shm.shm_frame->next;
					    dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
					  }
					}
				}
#endif /* FIX_FRAME */
				k++;
				if (k % frame_rate[system] == 0) { /* every second */
					printf("\n");
					fflush(stdout);
				}
				m = 0;
			}
			
			m++;
                        wbuf[vec].iov_base = (char *) dv;
                        wbuf[vec++].iov_len = DSIZE;

                        if (vec >= NPACKET_R) {
			  for(iovec_iter = 0; iovec_iter < vec; iovec_iter++) {
			    memcpy(&dvdata, wbuf[iovec_iter].iov_base, wbuf[iovec_iter].iov_len);
			    offset += wbuf[iovec_iter].iov_len;
			  }
			  
			  vec = 0;
			
			  if (offset == 144000) {
			    offset = 0;

			    _xdvshow_shm.shm_frame->frame_buf->lock = DVFRAME_DATA_READY;

#ifdef DEBUG
			    DPRINT("READ frame no. %d\n", sem_value++);
				if(sem_getvalue(&video_ready, &iDebugSemVal)) {
					perror("Unable to get value of video_ready semaphore!");
					exit(EXIT_FAILURE);
				}
				DPRINT("semaphore video_ready value: %d\n", iDebugSemVal);

				if(sem_getvalue(&video_empty, &iDebugSemVal)) {
					perror("Unable to get value of video_empty semaphore!");
					exit(EXIT_FAILURE);
				}
				DPRINT("semaphore video_empty value: %d\n", iDebugSemVal);
#endif /* DEBUG */

			    sem_post(&video_ready);
				ts_sem_wait(&video_empty);
			    
			    _xdvshow_shm.shm_frame = _xdvshow_shm.shm_frame->next;
			    dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
			  }
			}

		}
		ptr = (u_int32_t *)dv;
next:
		if ((char *)ptr < buf + tlen) {
			goto again;
		}

		if (vec > 0) {
		  for(iovec_iter = 0; iovec_iter < vec + 1; iovec_iter++) {
		    memcpy(&dvdata, wbuf[iovec_iter].iov_base, wbuf[iovec_iter].iov_len);
		    offset += wbuf[iovec_iter].iov_len;
		  }
		  if (offset == 144000) {
		    offset = 0;

		    _xdvshow_shm.shm_frame->frame_buf->lock = DVFRAME_DATA_READY;

#ifdef DEBUG
		    DPRINT("READ frame no. %d\n", sem_value++);
			if(sem_getvalue(&video_ready, &iDebugSemVal)) {
				perror("Unable to get value of video_ready semaphore!");
				exit(EXIT_FAILURE);
			}
			DPRINT("semaphore video_ready value: %d\n", iDebugSemVal);

			if(sem_getvalue(&video_empty, &iDebugSemVal)) {
				perror("Unable to get value of video_empty semaphore!");
				exit(EXIT_FAILURE);
			}
			DPRINT("semaphore video_empty value: %d\n", iDebugSemVal);
#endif /* DEBUG */

		    sem_post(&video_ready);
		    ts_sem_wait(&video_empty);
		    
		    _xdvshow_shm.shm_frame = _xdvshow_shm.shm_frame->next;
		    dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
		  }
		}
	}
	printf("\n");
	return NULL;

}
#endif /* FREEBSD_5 */

