/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** \mainpage <tt>xdvshow</tt> Documentation
 *
 * \section About What is xdvshow?
 *
 * <tt>xdvshow</tt> is a tool for displaying DV sent over IP using DVTS
 * standard on Linux and FreeBSD. It can use SDL display interface as well as
 * plain old X11 display interface. When using SDL, it can display video in
 * full-screen mode (either upscaled or with black frame around) and it is
 * ready for displaying stereo image as well.
 *
 * \section Usage How should I use it?
 *
 * Type <tt>xdvshow -h</tt> for getting help on command-line options. You can
 * get some more complex help by typing <tt>man xdvshow</tt>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>

#include "xdvshow-info.h"

/** \brief shows the xdvshow version.
 */
void
show_version(void)
{
  printf("xdvshow version %s\n", VERSION);
  printf("2003, 2004 Milos Liska (xliska@fi.muni.cz)\n");
  printf("This product includes software developed by Akimichi OGAWA.\n");
}


/** \brief prints a help message.
 */
void
show_usage(char *argv0)
{
  printf("Usage : %s [options]\n\n", argv0);

  printf("<options>\n");
  printf("-v        : show version number\n");
  printf("-h        : show this help message\n");
  printf("-f file   : input from file\n");
  printf("-A device : enable audio using device\n");
#if (HAVE_SDL || HAVE_SDL11)
  printf("-F        : enable fullscreen\n");
  printf("-z        : enable zoom (change display resolution)\n");
  printf("-Z        : enable zoom (scale video)\n");
  printf("-D        : enable deinterlace\n");
  printf("-o        : disable hw overlay with SDL\n");
#endif /* HAVE_SDL || HAVE_SDL11*/
  printf("-4        : use IPv4\n");
#ifdef	ENABLE_INET6
  printf("-6        : use IPv6\n");
#endif	/* ENABLE_INET6 */
  printf("-j addr   : join multicast group\n");
  printf("-r host   : packet reflector name/address\n");
  printf("-P port   : set UDP port number\n");
  printf("-S        : stereo\n");
#if (LINUX && HAVE_LIBRAW1394) || FREEBSD_5
  printf("-i        : use ieee1394 interface for DV acquisition\n");
#endif /* (DLINUX && HAVE_LIBRAW1394) || DFREEBSD_5 */
  printf("\n");
  printf("-d        : dump displayed dv data\n");
  printf("\n");
#if (HAVE_SDL || HAVE_SDL11)
  printf("-s        : use SDL for display (default)\n");
  printf("-x        : use X11 for display (no fs available)\n");
#else /* HAVE_SDL */
  printf("-x        : use X11 for display (no fs available, default)\n");
#endif

}
